﻿using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Dtos.Province.XieTong;
using DataSharing.Share.Dtos.Province.XieTong.Receive;
using DotNetCore.CAP;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using Quartz;
using SqlSugar;
using XF.Domain.Repository;

namespace DataSharing.Province.SendTask
{
    /// <summary>
    /// 定时拉取12315的过程数据
    /// </summary>
    public class TaskCaseDistrecordJob : IJob, IDisposable
    {
        private readonly ILogger<TaskCaseDistrecordJob> _logger;
        private readonly IRepository<DsSendCaseInfo> _sendCaseInfoRepository;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly XieTongClient _xieTongClient;
        private readonly IMapper _mapper;
        private readonly IRepository<DsGetCaseDistrecordSend> _getCaseDistrecordSendRepository;
        private readonly ICapPublisher _capPublisher;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="sendCaseInfoRepository"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="xieTongClient"></param>
        /// <param name="mapper"></param>
        /// <param name="getCaseDistrecordSendRepository"></param>
        /// <param name="capPublisher"></param>
        public TaskCaseDistrecordJob(ILogger<TaskCaseDistrecordJob> logger,
            IRepository<DsSendCaseInfo> sendCaseInfoRepository,
            IChannelConfigurationManager channelConfigurationManager,
            XieTongClient xieTongClient,
            IMapper mapper,
            IRepository<DsGetCaseDistrecordSend> getCaseDistrecordSendRepository,
            ICapPublisher capPublisher)
        {
            _logger = logger;
            _sendCaseInfoRepository = sendCaseInfoRepository;
            _channelConfigurationManager = channelConfigurationManager;
            _xieTongClient = xieTongClient;
            _mapper = mapper;
            _getCaseDistrecordSendRepository = getCaseDistrecordSendRepository;
            _capPublisher = capPublisher;
        }

        public async Task Execute(IJobExecutionContext context)
        {
            #region 定时拉取12315的数据
            //定时拉取12315的数据
            var listSendCaseInfo = await _sendCaseInfoRepository
                    .Queryable()
                    .Where(p => p.IsEnd == false)
                    .OrderBy(p => p.CreationTime)
                    .ToListAsync();
            //处理数据
            if (listSendCaseInfo.Any())
            {
                foreach (var itemData in listSendCaseInfo)
                {
                    GetCaseDistrecordSendInfo distrecordSendInfo = new() { CaseSerial = itemData.CaseSerial };

                    #region 处理请求参数
                    var configProvince = _channelConfigurationManager.GetConfigurationProvince();
                    var request = new ProvinceRequest<GetCaseDistrecordSendInfo>(configProvince.ClientId, configProvince.ClientSecret);
                    request.SetData(distrecordSendInfo);
                    var jsonData = request.ToString();

                    var baseAddress = configProvince.XieTong;
                    if (!baseAddress.EndsWith('/'))
                        baseAddress += "/";

                    var url = baseAddress + "rest/market/get_case_distrecord_send";
                    #endregion

                    var response = await _xieTongClient.RequestAsync<ProvinceDistrecordResponse>(url, "Post", jsonData, context.CancellationToken);

                    if (response is null || response.status is null || response.custom is null)
                        continue;

                    if (response.status.code != "1")
                        continue;

                    //这里处理业务
                    if (response.custom.STEP_List != null && response.custom.STEP_List.Count > 0)
                    {
                        //这里调用业务接口，推送办理数据
                        List<SharingWorkflowTraceDto> Traces = new();
                        //遍历返回的数据
                        foreach (var item in response.custom.STEP_List)
                        {
                            //检测当前数据是否存在，如果存在不处理，不存在新增到数据库并推送到业务系统
                            if (!await _getCaseDistrecordSendRepository.AnyAsync(p => p.CaseSerial == response.custom.CASE_SERIAL && p.RowGuid == item.RowGuid, context.CancellationToken))
                            {
                                var tempData = _mapper.Map<DsGetCaseDistrecordSend>(item);
                                await _getCaseDistrecordSendRepository.AddAsync(tempData);

                                Traces.Add(new()
                                {
                                    HandlerName = item.OperatorName,
                                    HandlerOrgName = item.OperatorouName,
                                    HandleTime = item.HandleTime,
                                    Opinion = item.HandleOpinion
                                });
                            }
                        }
                        SharingAddOrderTracesDto addOrderTracesDto = new()
                        {
                            OrderId = itemData.OrderID,
                            Traces = Traces
                        };
                        //往业务系统推送数据
                        await _capPublisher.PublishAsync(Hotline.Share.Mq.EventNames.SharingOrderAddTraces, addOrderTracesDto, cancellationToken: context.CancellationToken);
                    }

                    Thread.Sleep(2000);
                }
            }

            #endregion
            _logger.LogError($"进来了时间是{DateTime.Now}");
        }

        /// <summary>
        /// 
        /// </summary>
        public void Dispose()
        {

        }

    }
}
